#if !CONFIG_SOC_NRF5340_CPUAPP

#include <zephyr/kernel.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/sys/printk.h>
#include "nrf52832_board.h"
#include "error.h"
#include "gpio.h"

static const struct gpio_dt_spec _gAfe4900Reset = GPIO_DT_SPEC_GET(DT_ALIAS(afe4900reset), gpios);
static const struct gpio_dt_spec _gAfe4900Rdy   = GPIO_DT_SPEC_GET(DT_ALIAS(afe4900rdy),   gpios);
static const struct gpio_dt_spec _gPowerUp      = GPIO_DT_SPEC_GET(DT_ALIAS(powerup),      gpios);
static const struct gpio_dt_spec _gLedG         = GPIO_DT_SPEC_GET(DT_ALIAS(ledg),         gpios);

int gpioInit(void) {
    if (!device_is_ready(_gAfe4900Reset.port)) {
        printk("device %s is not ready\n", _gAfe4900Reset.port->name);
        return ERROR_GPIO_INITIALIZE_FAIL;
    }

    if (!device_is_ready(_gAfe4900Rdy.port)) {
        printk("device %s is not ready\n", _gAfe4900Rdy.port->name);
        return ERROR_GPIO_INITIALIZE_FAIL;
    }

    if (!device_is_ready(_gPowerUp.port)) {
        printk("device %s is not ready\n", _gPowerUp.port->name);
        return ERROR_GPIO_INITIALIZE_FAIL;
    }

    if (!device_is_ready(_gLedG.port)) {
        printk("device %s is not ready\n", _gLedG.port->name);
        return ERROR_GPIO_INITIALIZE_FAIL;
    }

    gpioConfigInput(AFE4900_RDY);
    gpioConfigOutput(AFE4900_RESET);
    gpioConfigOutput(POWER_UP);
    gpioConfigOutput(LED_G);
    gpioSet(POWER_UP);
    gpioClear(AFE4900_RESET);
    k_busy_wait(35);
    gpioSet(AFE4900_RESET);
    k_sleep(K_MSEC(10));

    return 0;
}

int gpioConfigInput(const uint32_t pin) {
    switch (pin) {
        case AFE4900_RDY:
        {
            const int ret = gpio_pin_configure_dt(&_gAfe4900Rdy, GPIO_INPUT);
            if (ret < 0) {
                printk("Failed to configure AFE4900_RDY pin as input, error: %d\n", ret);
                return ERROR_GPIO_CONFIG_FAIL;
            }
            printk("configure AFE4900_RDY pin as input\n");
        }
        break;
    }

    return 0;
}

int gpioConfigOutput(const uint32_t pin) {
    switch (pin) {
        case AFE4900_RESET:
        {
            const int ret = gpio_pin_configure_dt(&_gAfe4900Reset, GPIO_OUTPUT);
            if (ret < 0) {
                printk("Failed to configure AFE4900_RESET pin as output, error: %d\n", ret);
                return ERROR_GPIO_CONFIG_FAIL;
            }
            printk("configure AFE4900_RESET pin as output\n");
        }
        break;

        case POWER_UP:
        {
            const int ret = gpio_pin_configure_dt(&_gPowerUp, GPIO_OUTPUT);
            if (ret < 0) {
                printk("Failed to configure POWER_UP pin as output, error: %d\n", ret);
                return ERROR_GPIO_CONFIG_FAIL;
            }
            printk("configure POWER_UP pin as output\n");
        }
        break;
        
        case LED_G:
        {
            const int ret = gpio_pin_configure_dt(&_gLedG, GPIO_OUTPUT);
            if (ret < 0) {
                printk("Failed to configure LED_G pin as output, error: %d\n", ret);
                return ERROR_GPIO_CONFIG_FAIL;
            }
            printk("configure LED_G pin as output\n");
        }
        break;
    }

    return 0;
}

int gpioClear(const uint32_t pin) {
    switch (pin) {
        case AFE4900_RESET:
        {
            const int ret = gpio_pin_set_dt(&_gAfe4900Reset, 0);
            if (ret < 0) {
                printk("Failed to clear AFE4900_RESET pin, error: %d\n", ret);
                return ERROR_GPIO_SET_FAIL;
            }
            printk("clear AFE4900_RESET pin, %d\n", ret);
        }
        break;

        case POWER_UP:
        {
            const int ret = gpio_pin_set_dt(&_gPowerUp, 0);
            if (ret < 0) {
                printk("Failed to clear POWER_UP pin, error: %d\n", ret);
                return ERROR_GPIO_SET_FAIL;
            }
            printk("clear POWER_UP pin\n");
        }
        break;

        case LED_G:
        {
            const int ret = gpio_pin_set_dt(&_gLedG, 0);
            if (ret < 0) {
                printk("Failed to clear LED_G pin, error: %d\n", ret);
                return ERROR_GPIO_SET_FAIL;
            }
            printk("clear LED_G pin\n");
        }
        break;
    }

    return 0;
}

int gpioGet(const uint32_t pin) {
    switch (pin) {
        case AFE4900_RESET:
        {
            const int val = gpio_pin_get_dt(&_gAfe4900Reset);
            printk("get AFE4900_RESET pin, val: %d\n", val);
        }
        break;

        case AFE4900_RDY:
        {
            const int val = gpio_pin_get_dt(&_gAfe4900Rdy);
            printk("get AFE4900_RDY pin, val: %d\n", val);
        }
        break;

        case POWER_UP:
        {
            const int val = gpio_pin_get_dt(&_gPowerUp);
            printk("get POWER_UP pin, val: %d\n", val);
        }
        break;
    }

    return 0;
}

int gpioSet(const uint32_t pin) {
    switch (pin) {
        case AFE4900_RESET:
        {
            const int ret = gpio_pin_set_dt(&_gAfe4900Reset, 1);
            if (ret < 0) {
                printk("Failed to set AFE4900_RESET pin, error: %d\n", ret);
                return ERROR_GPIO_SET_FAIL;
            }
            printk("set AFE4900_RESET pin\n");
        }
        break;

        case POWER_UP:
        {
            const int ret = gpio_pin_set_dt(&_gPowerUp, 1);
            if (ret < 0) {
                printk("Failed to set POWER_UP pin, error: %d\n", ret);
                return ERROR_GPIO_SET_FAIL;
            }
            printk("set POWER_UP pin\n");
        }
        break;

        case LED_G:
        {
            const int ret = gpio_pin_set_dt(&_gLedG, 1);
            if (ret < 0) {
                printk("Failed to set LED_G pin, error: %d\n", ret);
                return ERROR_GPIO_SET_FAIL;
            }
            printk("set LED_G pin\n");
        }
        break;
    }

    return 0;
}

#endif