#if !CONFIG_SOC_NRF5340_CPUAPP

#include <zephyr/kernel.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/sys/printk.h>
#include "error.h"
#include "pwr_mgmt.h"

static const struct gpio_dt_spec _gPowerUp         = GPIO_DT_SPEC_GET(DT_ALIAS(powerup), gpios);
static power_state_t             _gPowerPowerState = POWER_STATE_DOWN;

int pwrMgmtInit(void) {
    // power up
    if (!device_is_ready(_gPowerUp.port)) {
        printk("Device %s is not ready\n", _gPowerUp.port->name);
        return ERROR_PWR_MGMT_INITIALIZE_FAIL;
    }

    const int ret = gpio_pin_configure_dt(&_gPowerUp, GPIO_OUTPUT);
    if (ret < 0) {
        printk("Failed to configure the LED pin, error: %d\n", ret);
        return ERROR_PWR_MGMT_INITIALIZE_FAIL;
    }

    _gPowerPowerState = POWER_STATE_DOWN;

    return 0;
}

int pwrMgmtPowerUp(void) {
    gpio_pin_set_dt(&_gPowerUp, 1);
    _gPowerPowerState = POWER_STATE_UP;

    return 0;
}

int pwrMgmtPowerDown(void) {
    gpio_pin_set_dt(&_gPowerUp, 0);
   _gPowerPowerState = POWER_STATE_DOWN;

    return 0;
}

power_state_t pwrMgmtGetPowerState(void) {
    return _gPowerPowerState;
}

#endif